<?php
/*
 * CClientscript extension. Adds auto Less compiling to registerCSSFile method
 * Based on LessCSS for Yii extension by zsoltlengyelit
 * orginal url: https://github.com/zsoltlengyelit/LessCSS-for-Yii
 * edited by André Kramer
 */
class LClientscript extends CClientScript{
    public $caching = true; //use cached less css file if available
    public $compress = false; //compress less css to 1 line

    public function registerCssFile($url,$media='')
    {
        //if the file extension is .less, use the lessparser.
        $file_extension = end(explode('.',$url));
        if($file_extension == 'less'){
            $url = $this->compileLess($url, false);
        }

		//run the parent method to parse the less/css file like Yii normally does.
        return parent::registerCssFile($url, $media);
    }

    protected function compileLess($less_input, $returnString = true)
    {
		//webroot of the application
        $basepath = Yii::getPathOfAlias('webroot')  . DIRECTORY_SEPARATOR;

        //store in runtime folder
        $cachePath = Yii::app()->runtimePath . DIRECTORY_SEPARATOR . 'minScript';
        if (!is_dir($cachePath)) {
            mkdir($cachePath, 0777, true);
        } else if (!is_writable($cachePath)) {
            throw new CException('LClientscript: ' . $cachePath . ' is not writable.');
        }

		//path to assets (default /assets)
		$assetspath = $cachePath . DIRECTORY_SEPARATOR;
        $assetName = md5($less_input);

        //reference where to store the temporary css file, before sending it to registerCSSFile
        $assetCssOrig = $assetspath . $assetName . '.css';

        $parsed = false;
		//only parse the file if caching is off and the file doesn't exist
		//Todo: Perhaps a check if the content is altered (date_modified) or file size changed
        //Todo: Use pathinfo to get the absolute url

        if(!$this->caching || !is_file($assetCssOrig)){
			//make the path absolute, but make sure basepath isn't in the url first.
            $sourcepath = $basepath . str_replace($basepath, '', $less_input);
            $sourcepath = str_replace('\\', '/', $sourcepath);
            $sourcepath = str_replace('//', '/', $sourcepath);

            if (is_file($sourcepath)) {
				//start parsing the less file and store it in assets folder
                file_put_contents($assetCssOrig, $this->parseLess($sourcepath));
            }
            else {
                throw new CException(__CLASS__.': Less stylesheet not found: '. $sourcepath);
            }
        }

        // return the parsed css string or file
        if ($returnString) {
            return $parsed;
        } else {
            return Yii::app()->getAssetManager()->publish($assetCssOrig);
        }
    }

    protected function parseLess($input)
    {
        $source = file_get_contents($input);
        $sourcedir = dirname($input);

        try {
			//include lessc library
            require_once(dirname(__FILE__) . DIRECTORY_SEPARATOR . 'include'. DIRECTORY_SEPARATOR . 'lessc.inc.php');

			//setup the parser
            $lessc = new lessc();
			if($this->compress){
				$lessc->setFormatter("compressed");
			}
			$lessc->importDir = $sourcedir; // Set the correct context when using @import

			//parse the file
            $parsed = $lessc->parse($source);
        } catch (exception $e) {
			throw new CException(__CLASS__.': Failed to compile less file with message: '.$e->getMessage().'.');
        }
        return $parsed;
    }
}